function [E, gradient, hessian] = compute_norm( f, q, nonzero_W, nz_W_idx, from, to, unlabeled_idxs, label_idxs, labels )
%COMPUTE_NORM The function optimized in the p-voltages algorithm.
%
%  Instead of minimizing the quadratic energy function, we want to
%  optimize an energy function with a power other than p=2. This function
%  computes that energy value. q is defined as 1/p + 1/q = 1.
%  This function is optimized for speed.
%
%  Input:
%    f: a column vector of labels for all unlabeled (not source/sink)
%       points
%    q: the q-value of the operation (related to p in p-resistances)
%
%   The following parameters are used for every evaluation and so should be
%   precomputed by the user beforehand.
%
%   nonzero_W: an array of nonzero edge weights in the graph. If W is
%     the weight matrix, this can be computed by W(find(W > 0)).
%
%   nz_W_idx: an array of indices of the nonzero weighted edges. This can
%     be computed as find(W > 0).
%
%   from is an array of indices of noes which connected to the nonzero
%   edges. to is a similar array. These can be computed as [from, to] =
%   find(W > 0).
%
%   unlabeled idxs is an array of indices of points which are unlabeled
%
%   label_idxs is an array of indices where the labeled points should be
%   placed
%
%   labels is an array (same length as label_idxs) which contains the
%   labels of the sources and sinks
%   The function outputs the energy E, the gradient, and the Hessian of the
%   function at the given value of f.

% Compute the value of the norm
num_labeled = length(labels);
full_f = zeros(length(f) + num_labeled, 1) + NaN;
full_f(label_idxs) = labels;
full_f(isnan(full_f)) = f;
N = length(full_f);
F = repmat(full_f, 1, N);
F = F - F';
selected = F(nz_W_idx);
Fe = nonzero_W.*(abs(selected).^q);
E = sum(Fe)/2;

% Compute the value of the norm's gradient
Fg = (sign(selected).*q.*nonzero_W).*(abs(selected).^(q-1));
gradient = accumarray(from, Fg);
gradient = gradient + accumarray(to, -Fg);
gradient = (gradient(unlabeled_idxs)./2)';

hessian = zeros(N,N);
hessian(nz_W_idx) = -q.*(q-1).*nonzero_W.*(abs(selected).^(q-2));
hessian = hessian + diag(-sum(hessian));
hessian = hessian(unlabeled_idxs, unlabeled_idxs);
end

